#!/usr/bin/env python
#

from gimpfu import *

############ 
# M A I N  #
#----------# 

def ColouredDoubleEngrave (inImage, inDrawable, basepat, slope, option, doFlatten) :
    
    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)
    
    # Get the layer position and set off visibility of other layers.
    pos = 0;
    for i in range(len(inImage.layers)):
        if(inImage.layers[i] == inDrawable):
            pos = i
        else:
            pdb.gimp_item_set_visible (inImage.layers[i], False)
     
    width = inDrawable.width
    height = inDrawable.height
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    if slope==0:
        EngraveParm = int(width/50)
    elif slope==90:
        EngraveParm = int(height/50)
    else:
        EngraveParm = int((width+height)/100)
    if  EngraveParm>60: # it seems this is a limit for engraving
        EngraveParm = 60
        
    FirstEngrave = inDrawable.copy()
    inImage.add_layer (FirstEngrave, pos+1)
    pdb.gimp_layer_add_alpha (FirstEngrave)
    FirstEngrave.mode = NORMAL_MODE
    FirstEngrave.name = "FirstEngrave: Image Layer for Horizontal Engrave Effect"
    FirstEngrave.opacity = 100.0
    rotation = (slope/180.0)*3.14159 	#converting to radians
    rotatedforw=pdb.gimp_item_transform_rotate (FirstEngrave, rotation, True, width/2, height/2)
    pdb.plug_in_engrave (inImage, FirstEngrave, EngraveParm, True)
    # rotate back
    rotation = (-slope/180.0)*3.14159 	#converting to radians
    rotatedback=pdb.gimp_item_transform_rotate (FirstEngrave, rotation, True, width/2, height/2)
    pdb.plug_in_spread (inImage, FirstEngrave, 0.25, 2.25)
    VERTICAL=1
    pdb.plug_in_ripple (inImage, FirstEngrave, 100, 1, VERTICAL, 0, 1, 0, 0)
    pdb.gimp_image_select_color (inImage, CHANNEL_OP_REPLACE, FirstEngrave, (0,0,0,255))
    pdb.gimp_context_set_foreground ((60,20,10,255))
    pdb.gimp_edit_fill (FirstEngrave, FOREGROUND_FILL)
    pdb.gimp_selection_none (inImage)
    
    SecondEngrave = inDrawable.copy()
    inImage.add_layer (SecondEngrave, pos+1)
    pdb.gimp_layer_add_alpha (SecondEngrave)
    SecondEngrave.mode = BURN_MODE
    SecondEngrave.name = "SecondEngrave: Image Layer for Vertical Engrave Effect"
    SecondEngrave.opacity = 30.0
    pdb.gimp_brightness_contrast (SecondEngrave, +90, +45)
    rotation = (slope/180.0)*3.14159 	#converting to radians
    rotatedforw=pdb.gimp_item_transform_rotate (SecondEngrave, rotation, True, width/2, height/2)
    pdb.plug_in_engrave (inImage, SecondEngrave, EngraveParm, True)
    # rotate back
    rotation = (-slope/180.0)*3.14159 	#converting to radians
    rotatedback=pdb.gimp_item_transform_rotate (SecondEngrave, rotation, True, width/2, height/2)
    pdb.plug_in_spread (inImage, SecondEngrave, 1.25, 0.12)
    HORIZONTAL=0
    pdb.plug_in_ripple (inImage, SecondEngrave, 100, 1, HORIZONTAL, 0, 1, 0, 0)
    pdb.gimp_image_select_color (inImage, CHANNEL_OP_REPLACE, SecondEngrave, (0,0,0,255))
    pdb.gimp_context_set_foreground ((10,4,1,255))
    pdb.gimp_edit_fill (SecondEngrave, FOREGROUND_FILL)
    pdb.gimp_selection_none (inImage)
    
    Colouring = inDrawable.copy()
    inImage.add_layer (Colouring, pos+0)
    Colouring.mode = DISSOLVE_MODE
    Colouring.name = "Colouring: Image Layer for more Colouring"
    Colouring.opacity = 22.0
    pdb.gimp_brightness_contrast (Colouring, 0, +25)
    if option == True :
        pdb.gimp_hue_saturation (inDrawable, 0, 0, 0, +50)
        pdb.gimp_hue_saturation (Colouring, 0, 0, 0, +100)
        
    inDrawable.mode = COLOR_MODE
    inDrawable.opacity = 100.0 
    
    ReinforceEngrave = SecondEngrave.copy()
    inImage.add_layer (ReinforceEngrave, pos+2)
    ReinforceEngrave.mode = BURN_MODE
    ReinforceEngrave.name = "ReinforceEngraveEngrave: 2nd Engrave Layer copied for reinforcing Effect"
    ReinforceEngrave.opacity = 100.0
    pdb.plug_in_dilate (inImage, ReinforceEngrave, 0, 3, 1.0, 15, 0, 255)
    
    (ww, wh, wbpp) = pdb.gimp_pattern_get_info (basepat)
    Material = pdb.gimp_layer_new (inImage, ww, wh, RGBA_IMAGE, "Material", 33.0, HARDLIGHT_MODE)     
    inImage.add_layer (Material, pos+2)
    pdb.gimp_context_set_pattern (basepat)
    pdb.gimp_edit_fill (Material, PATTERN_FILL)
    pdb.gimp_layer_scale (Material, width, height, False)
    
    # closing
    if doFlatten == True:
        Outcome = pdb.gimp_image_merge_visible_layers (inImage, CLIP_TO_BOTTOM_LAYER)
        Outcome.name = name+" Outcome of DoubleEngrave"
        # restore visibility to all layers  
        for i in range(len(inImage.layers)):
            if(inImage.layers[i] <> Outcome):
                pdb.gimp_item_set_visible (inImage.layers[i], True)
    else:
        Outcome = pdb.gimp_layer_new_from_visible (inImage, inImage, name+" Outcome of Double Engrave")
        inImage.add_layer (Outcome, pos+0)
    
    pdb.gimp_image_undo_group_end(inImage)
    pdb.gimp_displays_flush()
    return()
    
register(
    "ColouredDoubleEngrave",    "ColouredDoubleEngrave",    "ColouredDoubleEngrave",
    "Diego",   "Diego Nassetti ",   "2017",
    "ColouredDoubleEngrave",
    "RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_PATTERN, "base", "Base pattern:", 0),
      (PF_ADJUSTMENT, "slope", "Slope angle:", 0, (0, 180, 5)),
      (PF_TOGGLE, "saturate", "Saturate Outcome?", True),
      (PF_TOGGLE, "flatten", "Flatten Outcome?", True),
    ],
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    ColouredDoubleEngrave,
    menu="<Image>/Diego/Ancillary",
    )

main()
